'***********************************************************************************
'*  Name    : Garden timer remote.BAS                                          
'*  Author  : Frank Sloep                                       
'*  Notice  : Copyright (c) 2005                                
'*          : All Rights Reserved                               
'*  Date    : 23-1-05                                           
'*  Version : 1                                                 
'*  Notes   : GardenTimer Remote controller                
'*
'*
'* Connections to 16F628                                       
'*                                                              
;*           adres pin | A2        A1 | adres pin                
;*           adres pin | A3        A0 | nc              
;*           adres pin | A4        A7 | RF transmitter              
;*           adres pin | A5        A6 | Led              
'*              Ground |              | + 5V                       
;*            Button 1 | B0        B7 | Button 6                        
;*            Button 2 | B1        B6 | nc                        
;*            Button 3 | B2        B5 | nc               
;*            Button 4 | B3        B4 | Button 5                       
;*                                                              
; **********************************************************************************

Device 16F628
Config INTRC_OSC_NOCLKOUT, MCLRE_OFF, WDT_off, pwrte_on, BODEN_off, LVP_OFF
XTAL = 4
CMCON = 7           ' Comparators Off
VRCON = 0           ' Vref Off

Symbol Led PORTA.6  ' led
Dim DBYTE  As Byte  ' Holds 8-bit data byte
Dim ABYTE  As Byte  ' Holds 5-bit hardware address
Dim freq   As Byte  ' counter
Dim Zzzz   As Word  ' counter

On_Interrupt ISR          ' interrupt service routine
Set INTCON.5              ' Timer0 interrupt on
OPTION_REG = %00000110    ' prescaler to 128
TMR0 = 96                 ' (256-96)*127= 20.320ms led = (freq=250)*20ms=5sec
                          ' (256-96)*127= 20.320ms sleep = (Zzzz=1000)*20ms=20sec

 DelayMS 500          ' Stabilize on power-up
 TRISB = $FF          ' PORTB = 8-bit switch inputs
 PORTB_PULLUPS = On   ' PORTB internal pull-ups = on
 PORTA.7 = 0          ' RF Transmitter 
 TRISA = %00011111    ' PORTA.7/6/5 = output RF & LED & nop. Rest inputs for address
 Set INTCON.7         ' Global Interrups Enabled on (GIE)
 GoTo Encode

SEND:
  ABYTE = (PORTA >> 1) & %00011111     ' ABYTE = address from RA.1 to RA.5
  SerOut PORTA.7, 16780, [ABYTE,DBYTE] ' Transmit Abyte & Dbyte
  DelayMS 50                           
  Clear Zzzz
Encode:
  If Zzzz = 1000 Then GoTo Time_To_Sleep
  If freq=10 Or freq=30 Or freq=50 Then High Led
  If freq=20 Or freq=40 Or freq=60 Then Low Led
  If freq = 250 Then freq = 0

  If PORTB != $FF Then    ' Loop until port b key-press
    DBYTE = PORTB         ' DByte is data from port.b
    GoTo SEND             ' 
  EndIf
  GoTo Encode             ' start all over again
End   

Time_To_Sleep:
  Clear INTCON.7          ' GIE off
  Set INTCON.3            ' on change interrupt bit portb.4-7
  Low Led                 ' led on .. now off
  nap 7                   ' sleep and wait .... for a change on portb.4-7
  Zzzz = 0                ' wakeup & clear the sleep counter
  freq = 0                ' clear led counter
  Clear INTCON.0          ' clear change interrupt flag portb.4-7
  Clear INTCON.3          ' off Interrupt portb.4-7
  OPTION_REG = %00000110  ' set prescaler 128 after wakeup pic goes to prescaler 1:1
  Set INTCON.7            ' GIE on
  GoTo Encode             ' start all over again

ISR:
  Context save            ' 
  Inc Zzzz                ' up counter
  Inc freq                ' up Counter
  TMR0 = 96               ' init TMR0
  Clear INTCON.2          ' clear TMR0 overflow bit
  Context Restore         '
  GoTo Encode             ' start all over again





