'***********************************************************************************
'*  Name    : Garden timer.BAS                                          
'*  Author  : Frank Sloep                                       
'*  Notice  : Copyright (c) 2003 - 2005                                
'*          : All Rights Reserved                               
'*  Date    : 23-1-05                                           
'*  Version : 3                                                 
'*  Notes   : GardenTimer with extra output (relay 4) for waterpump (??)                
'*          : 4 modes                                           
'*          :   mode 1 always off                                      
'*          :   mode 2 Always on                                       
'*          :   mode 3 from dusk to dawn                                     
'*          :   mode 4 from dusk to alarm 1                                
'*          : extra alarm 2 for relay 4 to turn it off                                            '*          : RF remote controll                                                 
'*          : reprogram 5 bits hardware-adress                                                    
'*          : Delay-time for dusk and dawn from 0 to 20 minute                                                    
'*                                                              
'* Connections to 16F876a                                       
'*                                                              
;*                     | MCLR      B7 | Lcd D7              
;*                Mode | A0        B6 | Lcd D6              
;*            Settings | A1        B5 | Lcd D5              
;*                hour | A2        B4 | Lcd D4              
;*              Minute | A3        B3 | Lcd EN                   
;*      Relay 4 on/off | A4        B2 | Lcd RS                    
;*                  nc | A5        B1 | Backlicht                      
;*              Ground | -         B0 | nc                      
;*                     | Osc 1      - | Ground                        
;*                     | Osc 2      + | + 5V                         
;*             Relay 1 | C0        C7 | RF               
;*             Relay 2 | C1        C6 | ldr                      
;*             Relay 3 | C2        C5 | I2C SDA                 
;*             Relay 4 | C3        C4 | I2C SCL                
;*                                                              
; **********************************************************************************
Declare BOOTLOADER = off
Device 16F876A
Config hs_osc,wdt_off,pwrte_on,BODEN_On,LVP_OFF

ADCON1 = 7  
Declare XTAL = 10
Declare SLOW_BUS 0
Declare LCD_TYPE 0
Declare LCD_DTPIN PORTB.4
Declare LCD_ENPIN PORTB.3
Declare LCD_RSPIN PORTB.2
Declare LCD_INTERFACE 4
Declare LCD_LINES 2

Symbol Button1   PORTA.0   ; mode
Symbol Button2   PORTA.1   ; settings
Symbol Button3   PORTA.2   ; hour - backlight
Symbol Button4   PORTA.3   ; minute
Symbol Button5   PORTA.4   ; relay 4
Symbol ldr       PORTC.6   ; ldr
Symbol backlight PORTB.1   ; backlight LCD
Symbol Relay1    PORTC.0   ; Relay 1
Symbol Relay2    PORTC.1   ; Relay 2
Symbol Relay3    PORTC.2   ; Relay 3
Symbol Relay4    PORTC.3   ; Relay 4
Declare SDA_PIN  PORTC.5   ; I2C SDA
Declare SCL_PIN  PORTC.4   ; I2C SCL

TRISC=%11110000
TRISB=%10000000

Symbol Cont1 %11010000        ; write adres DS1307
Symbol Cont2 %11010001        ; read  adres DS1307
Dim timer3_on          As Bit ; 
Dim relay4_on          As Bit ;
Dim on_screen          As Bit ;
Dim ldr_pin            As Bit ;
Dim ldr_pin_old        As Bit ;
Dim backlight_timer_on As Bit ;
Dim LDR_on             As Bit ;
Dim relay1_activ       As Bit ;
Dim relay2_activ       As Bit ;
Dim relay3_activ       As Bit ;

Dim uur              As Byte  ; Time 10:00:00
Dim minuut           As Byte  ; Time 00:10:00
Dim seconden         As Byte  ; Time 00:00:10
Dim write_uur        As Byte  ; write to eeprom - once a hour
Dim delay            As Byte  ; Delaytime
Dim delay1           As Byte  ; Delay to turn on/off relays
Dim mode             As Byte  ; Menus
Dim settings         As Byte  ; Menus
Dim alarm1_uur       As Byte  ' Alarm time for relay 1/2/3
Dim Alarm1_min       As Byte  ' Alarm time for relay 1/2/3
Dim alarm2_uur       As Byte  ' Alarm time for relay 4
Dim alarm2_min       As Byte  ' Alarm time for relay 4
Dim Alarm1_mode      As Byte  ' start var for delay
Dim timer3_          As Byte
Dim timer_backlight  As Byte
Dim up_min           As Byte
Dim up_uur           As Byte
Dim temp             As Byte
Dim temp_            As Byte
Dim Alarm1_delay     As Byte
Dim x                As Byte
Dim onetime          As Byte
Dim memory           As Byte

'IR variable -------------------------------------------------------------------
Dim RF_Data       As Byte     ' Holds 8-bit data byte
Dim RF_Adres      As Byte     ' Holds 5-bit hardware address
Dim adr3          As Byte     ' Holds "Hardware" address
RF_Data = $FF
RF_Adres = %00000
'-------------------------------------------------------------------------------

adr3               = %11111    ; hardware address
delay              = 2         ; instelbaar~ delay in mode 3/4 licht/donker
delay1             = 2         ;
mode               = 1
settings           = 1
relay4_on          = 0
on_screen          = 1
alarm1_uur         = 0
Alarm1_min         = 0
alarm2_uur         = 0
alarm2_min         = 0
Alarm1_mode        = 0
timer3_            = 0       ; 5 sec pauze na mode
timer3_on          = 0       ; 
timer_backlight    = 0       ; backlight lcd
backlight_timer_on = 0       ; backlight lcd timer
ldr_pin_old        = 0       ; 
x                  = 0
onetime            = 0
relay1_activ       = 0
relay2_activ       = 0
relay3_activ       = 0
memory             = 0

DelayMS 500              ; wait 
;//////////////////////////////////////////////////////////////////////////////
;////// CUSTOM CHAR LCD////////////////////////////////////////////////////////
Print $fe, $40,14,17,17,17,14,10,14,0
Print $fe, $48,14,31,31,31,14,10,14,0
Print $fe, $50,6,12,24,24,24,12,6,0
Print $fe, $58,10,21,14,31,14,21,10,0
Print $fe, $60,3,2,26,10,10,12,8,0

;//////////////////////////////////////////////////////////////////////////////
;////// INIT DS1307 ///////////////////////////////////////////////////////////
BusOut Cont1, $00, [$00] ; sec to zerro
BusOut Cont1, $07, [$00] ; Oscillator enabled
BusOut Cont1, $02, [$00] ; 12/24 hour clock
DelayMS 10               ; wait 

;//////////////////////////////////////////////////////////////////////////////
;////// INIT MAINPROGRAM///////////////////////////////////////////////////////
 temp = ERead 0                                   ; read adres 0 eeprom
 If temp = 1 Then                                 ; if 0 first time 
   seconden = 0                                   ; put seconds to zero
   GoSub READ_FROM_EEPROM                         ; read from eeprom
   delay1 = delay                                 ; store delay-time
   If mode = 1 Then GoSub RELAY_OFF               ; turn Off relays If mode = 1
   If mode = 2 Then GoSub RELAY_ON                ; turn on relays if mode = 2
   If mode > 2 And LDR_on = 1 Then GoSub RELAY_ON ; and if mode > 2 and when the sun is down
   temp = minuut                                  ; store minute
   GoSub FIX_BCD_OUT                              ; dec to bcd
   BusOut Cont1,$01,[temp]                        ; write to ds1307
   temp = uur                                     ; ''
   GoSub FIX_BCD_OUT                              ; dec to bcd
   BusOut Cont1,$02,[temp]                        ; write to ds1307
   DelayMS 100                                    ; wait
 EndIf
 write_uur = uur + 1                              ; hour + 1 to store in Eeprom

;//
;//////////////////////////////////////////////////////////////////////////////
;////// MAIN SOURCE ///////////////////////////////////////////////////////////
  Cls                                             ; reset lcd
  Low backlight                                   ; backlight off

MAIN:
  If on_screen = 1 Then GoSub GET_TIME                 ; get time ds1307
  If on_screen = 1 Then GoSub TO_SCREEN                ; update LCD 
  If settings = 1 And Button1 = 0 Then GoSub MENU_MODE ; button 1 pressed

  ; Relay status after button1 = mode 
    If timer3_on = 1 And timer3_ = seconden Then  
      timer3_on = 0
      Alarm1_mode = 0
      If mode = 1 Then GoSub RELAY_OFF
      If mode = 2 Then GoSub RELAY_ON
      If mode > 2 Then
        If LDR_on = 1 Then 
		  GoSub RELAY_ON
		Else
          GoSub RELAY_OFF
		EndIf 
      EndIf
    EndIf

; button2 pressed
  If Button2 = 0  Then                               
    settings = settings + 1
    Print $fe, 1
    on_screen = 0
    temp = 1
    GoTo MENU_SET
  EndIf 

; button 3  - backlight
;           - hour - hour alarm 1 - hour alarm 2 - > pos hardware adres
; button 4  - min -  min alarm 1 - min alarm 2 - delay - hardware adres - memory reset

  If settings > 1 And Button3 = 0 Then GoTo MENU_UUR     
  If settings > 1 And Button4 = 0 Then GoTo MENU_MINUUT   
  If settings = 1 And Button3 = 0 Then GoSub menu_backlight
  
; Relay 4 on/off button 5
  If Button5 = 0 Then                                
    GoSub menu_backlight
    If Relay4 = 1 Then 
      Relay4 = 0
      relay4_on = 0
    Else
      Relay4 = 1
      relay4_on = 1
    EndIf
    DelayMS 300
  EndIf

; ldr check in mode 3/4
  If on_screen = 1 Then GoSub ldr_CHECK                   

; Alarm1 on? and time? turnoff relays!
  If alarm1_uur = uur And Alarm1_min = minuut And mode = 4 Then GoSub RELAY_OFF

; delay in mode 3 and 4 - and only when swap between daylight or moonlight 
  If Alarm1_mode = 1 And mode > 2 Then
    If Alarm1_delay <> minuut Then
      delay = delay - 1
      Alarm1_delay = minuut
    EndIf
  EndIf
  If delay < 0 Or delay > 20 Then
    Alarm1_mode = 0
    delay = delay1
    If mode = 3 Then
      If LDR_on = 1 Then 
        GoSub RELAY_ON
      Else
        GoSub RELAY_OFF
      EndIf
    EndIf
    If mode = 4 And LDR_on = 1 Then
        GoSub RELAY_ON
      EndIf
  EndIf
  
;Alarm 2 on? and time? turn relay 4 off
  If Relay4 = 1 Then
    If alarm2_uur = uur Then
      If alarm2_min = minuut Then
        Relay4 = 0
        relay4_on = 0
      EndIf
    EndIf
  EndIf

; Backlight timer 15 sec
  If backlight_timer_on = 1 Then                 ;backlight
    If timer_backlight = seconden Then
      backlight_timer_on = 0
      Low backlight
    EndIf
  EndIf

' Decode ir signals  
  GoTo RF_Decoder
    
; back to start
  GoTo MAIN
End

'/////////////////////////////////////////////////
'//////writings to LCD

TO_SCREEN:
  Print $fe, 2, DEC2 uur, ":", DEC2 minuut, ":", DEC2 seconden, " ", DEC2 delay, "    "
  If LDR_on = 1 Then
    Print $02
  Else
    Print $03
  EndIf
  Print $fe, $C0, "Mode ", DEC1 mode
;  Print $fe, $C0, DEC1 mode, " ", BIN5 RF_Adres, HEX2 RF_Data
  If Relay1 = 1 Then 
    Print $FE, $C0 + 12, $01
  Else
      Print $FE, $C0 + 12, $00
  EndIf
  If Relay2 = 1 Then 
    Print $FE, $C0 + 13, $01
  Else
    Print $FE, $C0 + 13, $00
  EndIf
  If Relay3 = 1 Then 
    Print $FE, $C0 + 14, $01
  Else
    Print $FE, $C0 + 14, $00
  EndIf
  If Relay4 = 1  Then
    Print $FE, $C0 + 15, $01
  Else
    Print $FE, $C0 + 15, $00
  EndIf
Return

'/////////////////////////////////////////////////
'//////Modes & settings

MENU_MODE:
  GoSub menu_backlight
  temp = mode
  Select Case mode
    Case 1
      mode = 2
	  DelayMS 200
    Case 2
      mode = 3
	  DelayMS 200
    Case 3
      mode = 4
	  DelayMS 200
    Case 4
      mode = 1
	  DelayMS 200
  End Select
  timer3_ = seconden + 5
  If timer3_ > 59 Then timer3_ = timer3_ - 59
  timer3_on = 1
Return

MENU_SET:
  GoSub menu_backlight
  Select Case settings
    Case 2
      Print $fe, 2, DEC2 uur, ":", DEC2 minuut
      Print $fe, $C0, "Clock  "
	  DelayMS 150
    Case 3
      Print $fe, 2, DEC2 alarm1_uur, ":", DEC2 Alarm1_min
      Print $fe, $C0, "Alarm 1"
	  DelayMS 150
    Case 4
      Print $fe, 2, DEC2 alarm2_uur, ":", DEC2 alarm2_min
      Print $fe, $C0, "Alarm 2"
	  DelayMS 150
    Case 5
      Print $fe, 2, DEC2 delay, "          "
      Print $fe, $C0, "Delay "
	  DelayMS 200
    Case 6
      If onetime = 0 Then 
        onetime = 1
        x = 0
      EndIf
      Print $fe, 2, "R1 - R2 - R3"
      If relay1_activ = 1 Then Print $fe, $C0,"On " : Else : Print $fe, $C0,"Off" : EndIf
      If relay2_activ = 1 Then Print $fe, $C0 + 5,"On " : Else : Print $fe, $C0 + 5,"Off" : EndIf
      If relay3_activ = 1 Then Print $fe, $C0 + 10,"On " : Else : Print $fe, $C0 + 10,"Off" : EndIf
      Print $FE, $C0 + x
      Print $FE, $0E
	  DelayMS 200
    Case 7
      If onetime = 0 Then 
        onetime = 1
        x = 1
      EndIf
      Print $FE, 2, "%", BIN5 adr3, " $", HEX2 adr3
      Print $FE, $C0, "Hardware adress"
      Print $FE, $80 + x
      Print $FE, $0E
	  DelayMS 400
     Case 8
      Print $fe, 2, "Reset Memory          "
      If memory = 1 Then Print $fe, $C0, "Yes"
      If memory = 0 Then Print $fe, $C0, "No "
	  DelayMS 200
   Case 9
      Cls
      If memory = 1 Then 
        Print $fe, 2, "Reset Memory"
        EWrite 0, [0]
        EWrite 1, [0]
        EWrite 2, [0]
        EWrite 3, [0]
        EWrite 4, [0]
        EWrite 5, [0]
        EWrite 6, [0]
        EWrite 7, [1]
        EWrite 8, [0]
        EWrite 9, [0]
        EWrite 10,[0]
        EWrite 11,[0]
        EWrite 12,[0]
        EWrite 13,[relay1_activ]
        EWrite 14,[relay2_activ]
        EWrite 15,[relay3_activ]
        EWrite 16,[adr3]
        Print $fe, $C0, "Turn off power!!"
        Stop
      EndIf
      Print $fe, 1
      on_screen = 1
      settings = 1
      onetime = 0
      Print $FE, $0C
      GoSub Write_to_eeprom
      temp = minuut
      GoSub FIX_BCD_OUT
      BusOut Cont1,$01,[temp]
      temp = uur
      GoSub FIX_BCD_OUT
      BusOut Cont1,$02,[temp]
	  DelayMS 400
    End Select
  DelayMS 100
GoTo MAIN

MENU_UUR:
  GoSub menu_backlight
  If settings = 2 Then up_uur = uur 
  If settings = 3 Then up_uur = alarm1_uur
  If settings = 4 Then up_uur = alarm2_uur

  Inc up_uur

  If up_uur = 24 Then up_uur = 0
  If settings = 2 Then uur = up_uur   
  If settings = 3 Then alarm1_uur = up_uur
  If settings = 4 Then alarm2_uur = up_uur
  If settings = 6 Then x = x + 5 : Print $FE, $C0 + x : If x > 11 Then x = 0 : EndIf
  If settings = 7 Then x = x + 1 : Print $FE, $80 + x : If x = 6 Then x = 1 : EndIf
  GoTo MENU_SET

MENU_MINUUT:
  GoSub menu_backlight
  If settings = 2 Then 
    Inc minuut
    If minuut > 60 Then minuut = 0
  EndIf
  If settings = 3 Then 
    Alarm1_min = Alarm1_min + 15
    If Alarm1_min > 45 Then Alarm1_min = 0
  EndIf
  If settings = 4 Then
    alarm2_min = alarm2_min + 15
    If alarm2_min > 45 Then alarm2_min = 0
  EndIf
  If settings = 5 Then
    Inc delay
    If delay > 15 Then delay = 0 
    delay1 = delay
  EndIf
  If settings = 6 Then
    If x = 0 Then 
      relay1_activ = relay1_activ ^ 1
      If relay1_activ = 0 Then Relay1 = 0
      If relay1_activ = 1 And mode > 1 Then Relay1 = 1
    EndIf
    If x = 5 Then 
      relay2_activ = relay2_activ ^ 1
      If relay2_activ = 0 Then Relay2 = 0
      If relay2_activ = 1 And mode > 1 Then Relay2 = 1
    EndIf
    If x = 10 Then 
      relay3_activ = relay3_activ ^ 1
      If relay3_activ = 0 Then Relay3 = 0
      If relay3_activ = 1 And mode > 1 Then Relay3 = 1
    EndIf
  EndIf
  If settings = 7 Then
    If x = 1 Then adr3 = adr3 ^ %10000                
    If x = 2 Then adr3 = adr3 ^ %01000                
    If x = 3 Then adr3 = adr3 ^ %00100
    If x = 4 Then adr3 = adr3 ^ %00010
    If x = 5 Then adr3 = adr3 ^ %00001
  EndIf
  If settings = 8 Then 
    Inc memory
    If memory > 1 Then memory = 0 
  EndIf
  GoTo MENU_SET


'/////////////////////////////////////////////////
'////////BACKLIGHT LCD

menu_backlight:
  High backlight
  backlight_timer_on = 1
  timer_backlight = seconden + 15
  If timer_backlight > 59 Then timer_backlight = timer_backlight - 59
Return

'/////////////////////////////////////////////////
'////////TIME I2C

GET_TIME:
  BusIn Cont2,$00,[temp] ; seconds
  GoSub FIX_BCD_IN
  seconden = temp
  BusIn Cont2,$01,[temp] ; minetes
  GoSub FIX_BCD_IN
  minuut = temp
  BusIn Cont2,$02,[temp] ; hours
  GoSub FIX_BCD_IN
  uur = temp
  If write_uur = uur Then          ; write to eeprom cicle = hour
    write_uur = uur + 1
    GoSub Write_to_eeprom
  EndIf
  DelayMS 10
Return

FIX_BCD_IN:
  temp_=temp & 15
  temp=temp >> 4
  temp=temp_ + 10*temp
Return

FIX_BCD_OUT:
temp_ = temp/$A 
temp = temp - (temp_*$A) 
temp = temp + (temp_ <<4)
Return

'/////////////////////////////////////////////////
'/////// LDR

ldr_CHECK:
  If ldr = 0 Then
    ldr_pin = 0
    LDR_on = 0
  Else       
    ldr_pin = 1
    LDR_on = 1
  EndIf
  If ldr_pin <> ldr_pin_old Then
    If ldr = 0 Then
      If Alarm1_mode = 0 Then
        Alarm1_mode = 1
        ldr_pin_old = 0
        Alarm1_delay = minuut
      EndIf  
    Else
      If Alarm1_mode = 0 Then
        Alarm1_mode = 1
        ldr_pin_old = 1
        Alarm1_delay = minuut
      EndIf  
    EndIf
  EndIf
Return

'/////////////////////////////////////////////////
'//////// RF DECODER

RF_Decoder:
  SerIn PORTC.7, 16780, 250, MAIN, [RF_Adres, RF_Data]
  If RF_Adres = adr3 Then
    If RF_Data = $EF Then
      If Relay4 = 1 Then 
        Relay4 = 0
      Else 
        Relay4 = 1
      EndIf
      DelayMS 500
    EndIf
    If RF_Data = $FE Then
      Inc mode
      If mode > 4 Then mode = 1
      timer3_ = seconden + 5
      If timer3_ > 59 Then timer3_ = timer3_ - 59
      timer3_on = 1
      DelayMS 500
    EndIf
    If RF_Data = $FB Then
      settings = settings + 1
      Print $fe, 1
      on_screen = 0
      temp = 1
      GoTo MENU_SET
      DelayMS 500
    EndIf
    If RF_Data = $F7 Then 
      If settings > 1 Then GoTo MENU_UUR
      If settings = 1 Then GoSub menu_backlight
      DelayMS 500
    EndIf
    If RF_Data = $FD Then 
      If settings > 1 Then GoTo MENU_MINUUT
      DelayMS 500
    EndIf
  EndIf
GoTo MAIN


'/////////////////////////////////////////////////
'//////// RELAY MODES

RELAY_ON:
If mode = 1 Then Return 
  If relay1_activ = 1 Then 
    Relay1 = 1
    DelayMS 500
  EndIf
  If relay2_activ = 1 Then 
    Relay2 = 1
    DelayMS 500
  EndIf
  If relay3_activ = 1 Then 
    Relay3 = 1
    DelayMS 500
  EndIf
Return

RELAY_OFF:
  Relay1 = 0
  DelayMS 500
  Relay2 = 0
  DelayMS 500
  Relay3 = 0
  DelayMS 500
Return

'/////////////////////////////////////////////////
'//////// EEPROM

READ_FROM_EEPROM:
uur          = ERead  1
minuut       = ERead  2
alarm1_uur   = ERead  3
Alarm1_min   = ERead  4
alarm2_uur   = ERead  5
alarm2_min   = ERead  6
mode         = ERead  7
delay        = ERead  8
Relay1       = ERead  9
Relay2       = ERead 10
Relay3       = ERead 11
Relay4       = ERead 12
relay1_activ = ERead 13
relay2_activ = ERead 14
relay3_activ = ERead 15
adr3         = ERead 16
Return

Write_to_eeprom:
  EWrite 0, [1]
  EWrite 1, [uur]
  EWrite 2, [minuut]
  EWrite 3, [alarm1_uur]
  EWrite 4, [Alarm1_min]
  EWrite 5, [alarm2_uur]
  EWrite 6, [alarm2_min]
  EWrite 7, [mode]
  EWrite 8, [delay]
  EWrite 9, [Relay1]
  EWrite 10,[Relay2]
  EWrite 11,[Relay3]
  EWrite 12,[Relay4]
  EWrite 13,[relay1_activ]
  EWrite 14,[relay2_activ]
  EWrite 15,[relay3_activ]
  EWrite 16,[adr3]
Return

EData 0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0



          

